#!/bin/sh
# restarts with tclsh \
exec wish "$0" "$@"

#The program creates rhythm exercises in the form of abc
#music notation files (out.abc). The file is converted
#into common music notation in the form of a postscript
#file using abcm2ps. The file is converted into a MIDI
#file using abc2midi. Ghostscript (gs) is used to display
#the postscript file. TiMidity or winamp is used to play
#the MIDI file.

#
# Copyright (C) 1998-2006 Seymour Shlien
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Original page:
#      http://ifdo.pugmarks.com/~seymour/runabc/top.html


set VERSION "TkRhythm 0.76 2007-09-15"

set usefont "Arial 10 bold"
if {[string length $usefont] > 0} {
  option add *Font $usefont}


global rhythm
set rhythm(path_abcm2ps) abcm2ps
set rhythm(path_abc2midi) abc2midi
if {$tcl_platform(platform) == "windows"} {
  set rhythm(path_gs) "C:/Program Files/gs/gs8.60/bin/gswin32.exe"
  set rhythm(path_midiplayer) "C:/Program Files/Winamp/winamp.exe"
  set rhythm(path_savefolder) "C:/lessons"
  } else {
  set rhythm(path_gs) gs
  set rhythm(path_midiplayer) timidity
  set rhythm(path_savefolder) "../lessons"
  } 
set rhythm(bpm) 60
set rhythm(beats_per_bar) 4
set rhythm(ties) 1
set rhythm(temp_path) ""
set rhythm(tiefraction) 0.5
set rhythm(rhythm1_instrument) 0
set rhythm(rhythm2_instrument) 0
set rhythm(leader_instrument) 0
set rhythm(pitch1) 74
set rhythm(pitch2) 43
set rhythm(2voice) 0
set rhythm(lockconfig) 0
set rhythm(nosave) 0
set rhythm(repeatability) 0
# rhythm(lockconfig) 1 does not allow changing paths to executables
# rhythm(lockconfig) 2 does not allow saving tkrhythm.ini file
# rhythm(nosave) 1 does not allow saving pdf and MIDI files

proc make_temp_folder {} {
global rhythm
if {[string length $rhythm(temp_path)] > 1} {file mkdir $rhythm(temp_path)}
global rhythm
}


proc make_midi_pitch_translator {} {
global pitch2note
array set pitch2note {
        60 C    61 ^C   62 D    63 ^D   64 E    65 F
        66 ^F   67 G    68 ^G   69 A    70 ^A   71 B
        72 c    73 ^c   74 d    75 ^d   76 e    77 f
        78 ^f   79 g    80 ^g   81 a    82 ^a   83 b
        }
for {set i 60} {$i < 72} {incr i} {
        set i1 [expr $i -12]
        set i2 [expr $i -24]
        set i3 [expr $i- 36]
        set i4 [expr $i -48]
        set j [expr $i + 12]
        set j1 [expr $j +12]
        set j2 [expr $j +24]
        set j3 [expr $j +36]
        set j4 [expr $j +48]
        set pitch2note($i1) "$pitch2note($i),"
        set pitch2note($i2) "$pitch2note($i),,"
        set pitch2note($i3) "$pitch2note($i),,,"
        set pitch2note($i4) "$pitch2note($i),,,,"
        set pitch2note($j1) "$pitch2note($j)'"
        set pitch2note($j2) "$pitch2note($j)''"
        set pitch2note($j3) "$pitch2note($j)'''"
        set pitch2note($j4) "$pitch2note($j)''''"
        }
}

make_midi_pitch_translator

wm protocol . WM_DELETE_WINDOW {write_ini_file tkrhythm.ini; exit}


proc makebar {} {
#using a random number generator, the function creates
#a bar of music from the choice of rhythm elements.
set bar ""
set kount 0
while {$kount < 2} {
 set str ""
 set fac 3.0
 set units 0
 set left [expr 6 - $units]
 while {$units < 6} {
   if {$left > 1} {
     set n [expr $fac * rand() ]
     set n [expr int($n + 1.1)]
     } else {set n 1}
   if {$n > $left} {set n $left}
   append str G$n
   incr units $n
   set left [expr 6 - $units]
   if {$left < $fac} {set fac $left}
   #puts "$n $units $left $fac"
  }
 set bar "$bar $str"
 incr kount
 }
puts $bar
return $bar
}

#These are the rhythm elements, played as the key of G.

# compound time
array set b {
0 G2G2G2 1 G2G4 2 G4G2 3 GGG2G2 4 G2GGG2
5 G2G2GG 6 GGGGG2 7 GGG2GG 8 G2GGGG 9 GGGGGG
10 G3GG2 11 GG3G2 12 G2G3G 13 G2GG3 14 G3G3
15 zG2G3 16 z2GG3 17 z3G3 18 zG2GGG 19 z2GG2G}

# mark mubols which begin with a rest
array set b1z {
0 0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8 0 9 0
10 0 11 0 12 0 13 0 14 0 15 1 16 1 17 1 18 1 19 1 }


#mark mubols whic end with a rest
array set b2z {
0 0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8 0 9 0
10 0 11 0 12 0 13 0 14 0 15 0 16 0 17 0 18 0 19 0 }

 
#simple time
array set c {
0 G4 1 G2G2 2 GG3 3 G3G 4 GGGG
5 GGG2 6 G2GG 7 GG2G 8 z2G2 9 zG3
10 z2GG 11 zGGG 12 z4 13 z3G 14 zGG2
15 zG2G 16 (3G2G2G2 17 (3G2G2-G2 18 (3G2-G2G2 19 (3G2G2z2
20 (3G2z2G2 21 (3z2G2G2 }

# mark mubols which begin with a rest
array set c1z {
0 0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8 1 9 1
10 1 11 1 12 1 13 1 14 1 15 1 16 0 17 0 18 0 19 0
20 0 21 1 }

# mark mubols which end with a rest
array set c2z {
0 0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8 0 9 0
10 0 11 0 12 1 13 0 14 0 15 0 16 0 17 0 18 0 19 1
20 0 21 0 }


#We do not want a glyph file name beginning with an opening
#parenthesis, so we eliminate it for the triplets.
proc substitute_parenthesis {notes} {
 set first  1
 while {$first > -1} {
 set first [string first "(" $notes)]
 if {$first != -1} {
   set notes [string replace $notes $first $first t]}
 set first [string first " " $notes)]
 if {$first != -1} {
   set notes [string replace $notes $first $first ]}
 }
return $notes
}

proc make_simple_interface {} {
global c
global rhythmsel
set w .rhythmselector2
frame $w
for {set i 0} {$i <= 21} {incr i} {
 # if {$i == 11} continue
  set im [image create photo]
  set name [substitute_parenthesis $c($i)]
  $im read glyphs/$name.gif
  checkbutton $w.$i -image $im -variable rhythmsel($i) -borderwidth 3 -relief sunken
  set k [expr $i % 6]
  set r [expr $i / 6]
  grid $w.$i -row $r -column $k -sticky w
  }
make_random_selection 5 22
}

proc make_compound_interface {} {
global b
global rhythmsel
set w .rhythmselector3
frame $w
for {set i 0} {$i <= 19} {incr i} {
  set im [image create photo]
  set name [substitute_parenthesis $b($i)]
  $im read glyphs/$name.gif
  checkbutton $w.$i -image $im -variable rhythmsel($i) -borderwidth 3 -relief sunken
  set k [expr $i % 5]
  set r [expr $i / 5]
  grid $w.$i -row $r -column $k -sticky w
  }
}

proc make_random_selection {n max} {
global rhythmsel
for {set i 0} {$i < $n} {incr i} {
  set j [expr int($max*rand())]
  set rhythmsel($j) 1
  }
}
proc make_rhythm_list {} {
global rhythmsel
global rhythmlist
global timesigtype
set rhythmlist {}
for {set i 0} {$i <= 21} {incr i} {
 if {$timesigtype && $i>19} break
 if {$rhythmsel($i)} {
   lappend rhythmlist $i }
 }
} 


proc make_all_glyphs {} {
global c b
global out
for {set i 0} {$i <= 28} {incr i} {
 if {[expr $i % 5] != 0} { puts $out "$c($i) \\"
 } else {puts $out $c($i)}
}
for {set i 0} {$i <= 24} {incr i} {
  if {[expr $i % 5] != 0} { puts $out "$b($i) \\"
   } else {puts $out $b($i)}
  }
 
} 


proc substitute_note {line  newnote} {
 set start 0
 set newline ""
 while {[set loc [string first "G" $line $start]] > -1} {
#    puts "$start $loc"
    if {$loc == -1} break
    if {$loc == $start} {set frag $newnote
     set newline $newline$frag
     } else {
       set frag "[string range $line $start [expr $loc -1]]$newnote"
       set newline $newline$frag
       }
     set start [expr $loc+1]
   }
 set frag [string range $line $start end]
 set newline $newline$frag
# puts $line
# puts $newline
 return $newline
 }


proc make_compound_header {beats} {
global out
global rhythm
set divisions [expr $beats*3]
puts $out "X:1"
puts $out "T: $divisions/8 rhythm "
puts $out "M: $divisions/8"
puts $out "L: 1/16"
puts $out "K: none"
}



proc make_compound_leader {nbeats guitarchord} {
global outhandle
switch $nbeats {
 2 { puts $outhandle "$guitarchord z12|\\" }
 3 { puts $outhandle "$guitarchord  z18|\\" }
 4 { puts $outhandle "$guitarchord  z24|\\" }
  }
puts $outhandle "%%MIDI gchordoff"
}


proc make_simple_header {nbeats} {
global out
global rhythm
puts $out "X:1"
puts $out "T: $nbeats/4 rhythm"
puts $out "M: $nbeats/4"
puts $out "L: 1/16"
puts $out "Q: 1/4=$rhythm(bpm)"
puts $out "K: none"
}


proc make_simple_leader {nbeats guitarchord} {
global outhandle
switch $nbeats {
 4 { puts $outhandle "$guitarchord  z16|\\" }
 3 { puts $outhandle "$guitarchord z12|\\" }
 2 { puts $outhandle "$guitarchord z8|\\" }
 }
puts $outhandle "%%MIDI gchordoff"
}



proc make_simple_abc_rhythm {nbeats nbars nlines} {
global rhythm
global rhythmlist
global c c1z c2z
global out
make_rhythm_list
set n [llength $rhythmlist]

#we do it in to passes so we can insert ties in the right places

set mubol_list {}
set nmubols [expr $nlines*$nbars*$nbeats]
for {set i 0} {$i < $nmubols} {incr i} {
  set ni [expr int($n*rand())]
  set in [lindex $rhythmlist $ni]
  lappend mubol_list $in
  }

set index 0
for {set lines 1} {$lines <= $nlines} {incr lines} {
  for {set bars 1} {$bars <= $nbars} {incr bars} {
    set output ""
    for {set i 0} {$i < $nbeats} {incr i} {
      set in [lindex $mubol_list  $index]
      set output "$output $c($in)"
      if {$rhythm(ties)} {
         set tie [expr rand()]
         set index1 [expr $index + 1]
         set in1 [lindex $mubol_list $index1]
         if { $tie < $rhythm(tiefraction) && $index1 < $nmubols\
               && !$c2z($in) && !$c1z($in1)} {
		set output "$output-"}
         }
      incr index
      }
    if {$bars == $nbars} {puts $out "$output |"
              } else {puts $out "$output |\\"}
    }
  }
}
 

proc make_compound_abc_rhythm {nbeats nbars nlines} {
global rhythmlist
global b b1z b2z
global out
global rhythm
make_rhythm_list
set n [llength $rhythmlist]

set mubol_list {}
set nmubols [expr $nlines*$nbars*$nbeats]
for {set i 0} {$i < $nmubols} {incr i} {
  set ni [expr int($n*rand())]
  set in [lindex $rhythmlist $ni]
  lappend mubol_list $in
  }

set index 0
for {set lines 1} {$lines <= $nlines} {incr lines} {
  for {set bars 1} {$bars <= $nbars} {incr bars} {
    set output ""
    for {set i 0} {$i < $nbeats} {incr i} {
      set in [lindex $mubol_list $index]
      set output "$output $b($in)"
      if {$rhythm(ties)} {
         set tie [expr rand()]
         set index1 [expr $index + 1]
         set in1 [lindex $mubol_list $index1]
         if {$tie < $rhythm(tiefraction) && $index1 < $nmubols\
              && !$b2z($in) && !$b1z($in1)} {
           set output "$output-"}
         }
      incr index
      }
    if {$bars == $nbars} {puts $out "$output |"
              } else {puts $out "$output |\\"}
    }
  }
}
 


proc make_compound_abc {nbeats} {
global out
global rhythm
set out [open $rhythm(temp_path)out.abc w]
make_compound_header $nbeats 
puts $out "V:1 staff=perc stafflines=1"
#make_compound_leader $nbeats \"G\"
switch $nbeats {
 2 {make_compound_abc_rhythm $nbeats 4 4}
 3 {make_compound_abc_rhythm $nbeats 3 4}
 4 {make_compound_abc_rhythm $nbeats 2 4}
 }
close $out
}

proc make_2voice_compound_abc {nbeats} {
global out rhythm
set out [open $rhythm(temp_path)out.abc w]
make_compound_header $nbeats 
puts $out "V:1 staff=perc stafflines=1"
#make_compound_leader $nbeats \"G\"
switch $nbeats {
 2 {make_compound_abc_rhythm $nbeats 4 2}
 3 {make_compound_abc_rhythm $nbeats 3 2}
 4 {make_compound_abc_rhythm $nbeats 2 2}
 }
puts $out "V:2 staff=perc stafflines=1"
#make_compound_leader $nbeats ""
switch $nbeats {
 2 {make_compound_abc_rhythm $nbeats 4 2}
 3 {make_compound_abc_rhythm $nbeats 3 2}
 4 {make_compound_abc_rhythm $nbeats 2 2}
 }
close $out
}

proc make_simple_abc {nbeats} {
global out rhythm
set out [open $rhythm(temp_path)out.abc w]
make_simple_header $nbeats 
puts $out "V:1 staff=perc stafflines=1"
switch $nbeats {
 2 {make_simple_abc_rhythm $nbeats 4 4}
 3 {make_simple_abc_rhythm $nbeats 3 4}
 4 {make_simple_abc_rhythm $nbeats 2 4}
 }
close $out
}

proc make_2voice_simple_abc {nbeats} {
global out rhythm
set out [open $rhythm(temp_path)out.abc w]
make_simple_header $nbeats 
puts $out "V:1 staff=perc stafflines=1"
switch $nbeats {
 2 {make_simple_abc_rhythm $nbeats 4 2}
 3 {make_simple_abc_rhythm $nbeats 3 2}
 4 {make_simple_abc_rhythm $nbeats 2 2}
 }
puts $out "V:2 staff=perc stafflines=1"
switch $nbeats {
 2 {make_simple_abc_rhythm $nbeats 4 2}
 3 {make_simple_abc_rhythm $nbeats 3 2}
 4 {make_simple_abc_rhythm $nbeats 2 2}
 }
close $out
}

proc make_abc_file {} {
global timesigtype
global rhythm
set nbeats $rhythm(beats_per_bar)
if {$rhythm(2voice)} {
#  puts "rhythm(2voice) = $rhythm(2voice)"
  if {$timesigtype} {
              make_2voice_compound_abc $nbeats
       } else {make_2voice_simple_abc $nbeats}
  } else {
#  puts "rhythm(2voice) = $rhythm(2voice)"
  if {$timesigtype} {
              make_compound_abc $nbeats
       } else {make_simple_abc $nbeats}
  }
display_output
}


proc display_output {} {
global cmn
global rhythm
global result result2
global tcl_platform
if {![winfo exist .music]} {
  toplevel .music
  canvas .music.c -width 600 -height 300
  pack .music.c
  set cmn [image create photo -width 600 -height 300]
  }
set cmd "exec [list $rhythm(path_abcm2ps)]  $rhythm(temp_path)out.abc -O $rhythm(temp_path)Out.ps"
catch {eval $cmd} result
#puts "cmd = $cmd"
#puts "result = $result"
set result [split $result] ;# to get a valid list containing items in quotes
if {[string equal [lindex $result 0] "couldn't"]} { 
  tk_messageBox -type ok -icon info -message $result}
set cmd "exec [list $rhythm(path_gs)] -dBATCH -dNOPAUSE -sDEVICE=pgmraw -q -sOutputFile=$rhythm(temp_path)out.pgm $rhythm(temp_path)Out.ps "
catch {eval $cmd} result2
set result2 [split $result2] ;# to get a valid list containing items in quotes
if {[string equal [lindex $result2 0] "couldn't"]} { 
  tk_messageBox -type ok -icon info -message $result2
  if {$tcl_platform(platform) == "windows"} {
     tk_messageBox -type ok -icon info -message "You need to install\
 the ghostscript package if you have not done so. It is available from\n\
http://www.cs.wisc.edu/~ghost/"}
  }
$cmn read $rhythm(temp_path)out.pgm
.music.c create image 0 0 -anchor nw  -image $cmn
.control.play configure -state normal
.control.playmenu configure -state normal
set cmd "exec [list $rhythm(path_gs)] -dBATCH -dNOPAUSE -sDEVICE=pdfwrite -q -sOutputFile=$rhythm(temp_path)out.pdf $rhythm(temp_path)Out.ps "
eval $cmd
}


proc add_MIDI_info {novoice2} {
global rhythm
global outhandle
global timesigtype
global pitch2note
set voiceno 1
set note1 $pitch2note($rhythm(pitch1))
set note2 $pitch2note($rhythm(pitch2))
set inhandle [open $rhythm(temp_path)out.abc r]
set outhandle [open $rhythm(temp_path)outm.abc w]
while {![eof $inhandle]} {
  gets $inhandle line
  set code [string range $line 0 2]
  if {[string equal $code "V:1"]} {
    set voiceno 1
    puts $outhandle $line
    if {$timesigtype} {
      make_compound_leader $rhythm(beats_per_bar) \"G\"
     } else {
      make_simple_leader $rhythm(beats_per_bar) \"G\"
    }
    continue;
    }
  if {[string equal $code "V:2"]} {
    set voiceno 2
    if {$novoice2} {break}
    puts $outhandle $line
    puts $outhandle "%%MIDI program $rhythm(rhythm2_instrument)"
    if {$timesigtype} {
      make_compound_leader $rhythm(beats_per_bar) ""
     } else {
      make_simple_leader $rhythm(beats_per_bar) ""
    }
    continue
  }
 if {$voiceno == 1}  {puts $outhandle [substitute_note $line $note1]
   } else {puts $outhandle [substitute_note $line $note2]}
 
 set code [string range $line 0 1]
 if {[string equal $code "K:"]} {
     puts $outhandle "Q: 1/4=$rhythm(bpm)"
     include_play_info $outhandle}
  }
close $inhandle
close $outhandle
}

proc make_compound_gchord {nbeats handle} {
switch $nbeats {
 2 {puts $handle "%%MIDI gchord fz3fz3" }
 3 {puts $handle "%%MIDI gchord fz3fz3fz3" }
 4 {puts $handle "%%MIDI gchord fz3fz3fz3fz4" }
 }
}

proc make_simple_gchord {nbeats handle} {
switch $nbeats {
 4 { puts $handle "%%MIDI gchord fz7fz7fz7fz7" }
 3 { puts $handle "%%MIDI gchord fz7fz7fz7"}
 2 { puts $handle "%%MIDI gchord fz7fz7" }
  }
}


proc include_play_info {handle} {
global timesigtype rhythm
puts $handle "%%MIDI beat 100 100 100 6"
puts $handle "%%MIDI program $rhythm(rhythm1_instrument)"
puts $handle "%%MIDI bassprog $rhythm(leader_instrument)"
set nbeats $rhythm(beats_per_bar)
if {$timesigtype} {
  make_compound_gchord $nbeats $handle
  } else {
  make_simple_gchord $nbeats $handle
  }
}

proc play_exercise {onlyvoice1} {
global rhythm
global result result2
add_MIDI_info $onlyvoice1
set cmd "exec [list $rhythm(path_abc2midi)]  $rhythm(temp_path)outm.abc -o $rhythm(temp_path)outm1.mid"
catch {eval $cmd} result
#puts $result
set result [split $result] ;# to get a valid list containing items in quotes
if {[string equal [lindex $result 0] "couldn't"]} { 
  tk_messageBox -type ok -icon info -message $result}
if {[string length $rhythm(temp_path)] > 1} {
  set file outm1.mid
} else {set file [pwd]/outm1.mid}

set cmd "exec [list $rhythm(path_midiplayer)] $rhythm(temp_path)[list $file] &"
catch {eval $cmd} result2
set result2 [split $result2] ;# to get a valid list containing items in quotes
if {[string equal [lindex $result2 0] "couldn't"]} { 
  tk_messageBox -type ok -icon info -message $result2}
if {!$rhythm(nosave)} {.control.save configure -state normal}
}

proc save_pdf_midi {} {
global rhythm
file mkdir "$rhythm(path_savefolder)"
set savefile [tk_getSaveFile -initialdir $rhythm(path_savefolder)]
if {[string length $savefile] > 1} {
  .control.save configure -state disable
  set root [file rootname $savefile]
  file rename -force $rhythm(temp_path)out.pdf $root.pdf
  file rename -force $rhythm(temp_path)outm1.mid $root.mid}
}

proc set_savefolder {} {
global rhythm
set folder [tk_chooseDirectory]
if {[string length $folder] > 1} {
  set rhythm(path_savefolder) $folder
  }
file mkdir $folder
}


proc switch_type {} {
#switch between compound and simple rhythm
global timesigtype
 if {$timesigtype == 0} {pack forget .rhythmselector3
              pack .rhythmselector2 -after .control
 } else {pack forget .rhythmselector2;
         pack .rhythmselector3 -after .control}
}

proc read_patch_names {} {
global patches
array set patches {
0 "Acoustic Grand" 1 "Bright Acoustic" 2 "Electric Grand" 3 "Honky-Tonk"
4 "Electric Piano 1" 5 "Electric Piano 2" 6 "Harpsichord" 7 "Clav"
8 "Celesta" 9 "Glockenspiel" 10 "Music Box" 11 "Vibraphone"
12 "Marimba" 13 "Xylophone" 14 "Tubular Bells" 15 "Dulcimer"
16 "Drawbar Organ" 17 "Percussive Organ" 18 "Rock Organ" 19 "Church Organ"
20 "Reed Organ" 21 "Accordian" 22 "Harmonica" 23 "Tango Accordian"
24 "Acoustic Guitar (nylon)" 25 "Acoustic Guitar (steel)"
26 "Electric Guitar (jazz)" 27 "Electric Guitar (clean)"
28 "Electric Guitar (muted)" 29 "Overdriven Guitar" 30 "Distortion Guitar"
31 "Guitar Harmonics" 32 "Acoustic Bass" 33 "Electric Bass (finger)"
34 "Electric Bass (pick)" 35 "Fretless Bass" 36 "Slap Bass 1"
37 "Slap Bass 2" 38 "Synth Bass 1" 39 "Synth Bass 2" 40 "Violin"
41 "Viola" 42 "Cello" 43 "Contrabass" 44 "Tremolo Strings"
45 "Pizzicato Strings" 46 "Orchestral Strings" 47 "Timpani"
48 "String Ensemble 1" 49 "String Ensemble 2" 50 "SynthStrings 1"
51 "SynthStrings 2" 52 "Choir Aahs" 53 "Voice Oohs"
54 "Synth Voice" 55 "Orchestra Hit" 56 "Trumpet"
57 "Trombone" 58 "Tuba" 59 "Muted Trumpet" 60 "French Horn"
61 "Brass Section" 62 "SynthBrass 1" 63 "SynthBrass 2" 64 "Soprano Sax"
65 "Alto Sax" 66 "Tenor Sax" 67 "Baritone Sax" 68 "Oboe"
69 "English Horn" 70 "Bassoon" 71 "Clarinet" 72 "Piccolo"
73 "Flute" 74 "Recorder" 75 "Pan Flute" 76 "Blown Bottle"
77 "Skakuhachi" 78 "Whistle" 79 "Ocarina" 80 "Lead 1 (square)"
81 "Lead 2 (sawtooth)" 82 "Lead 3 (calliope)" 83 "Lead 4 (chiff)"
84 "Lead 5 (charang)" 85 "Lead 6 (voice)" 86 "Lead 7 (fifths)"
87 "Lead 8 (bass+lead)" 88 "Pad 1 (new age)" 89 "Pad 2 (warm)"
90 "Pad 3 (polysynth)" 91 "Pad 4 (choir)" 92 "Pad 5 (bowed)"
93 "Pad 6 (metallic)" 94 "Pad 7 (halo)" 95 "Pad 8 (sweep)"
96 "FX 1 (rain)" 97 "(soundtrack)" 98 "FX 3 (crystal)"
99 "FX 4 (atmosphere)" 100 "FX 5 (brightness)" 101 "FX 6 (goblins)"
102 "FX 7 (echoes)" 103 "FX 8 (sci-fi)" 104 "Sitar" 105 "Banjo"
106 "Shamisen" 107 "Koto" 108 "Kalimba" 109 "Bagpipe"
110 "Fiddle" 111 "Shanai" 112 "Tinkle Bell" 113 "Agogo"
114 "Steel Drums" 115 "Woodblock" 116 "Taiko Drum" 117 "Melodic Tom"
118 "Synth Drum" 119 "Reverse Cymbal" 120 "Guitar Fret Noise"
121 "Breath Noise" 122 "Seashore" 123 "Bird Tweet" 124 "Telephone ring"
125 "Helicopter" 126 "Applause" 127 "Gunshot" }
}



proc make_config_setup {} {
global rhythm
global patches
if {[winfo exist .setup]} return
set w .setup
toplevel .setup
button $w.abcm2ps -text abcm2ps -width 14 -command {setpath path_abcm2ps
    .setup.abcm2pse  xview moveto 1.0}
entry $w.abcm2pse -width 30 -relief sunken -textvariable rhythm(path_abcm2ps)
grid $w.abcm2ps $w.abcm2pse  
bind .setup.abcm2pse  <Return> {focus .setup}
.setup.abcm2pse  xview moveto 1.0
button $w.abc2midi -text abc2midi -width 14 -command {setpath path_abc2midi
.setup.abc2midie  xview moveto 1.0}
entry $w.abc2midie -width 30 -relief sunken -textvariable rhythm(path_abc2midi)
grid $w.abc2midi $w.abc2midie  
bind .setup.abc2midie  <Return> {focus .config}
.setup.abc2midie  xview moveto 1.0
button $w.gs -text "ghostscript (gs)" -width 14 -command {setpath path_gs
.setup.gse  xview moveto 1.0}
entry $w.gse -width 30 -relief sunken -textvariable rhythm(path_gs)
grid $w.gs $w.gse 
bind .setup.gse  <Return> {focus .config}
.setup.gse  xview moveto 1.0
button $w.midi -text "MIDI player" -width 14 -command {setpath path_midiplayer
.setup.midie  xview moveto 1.0}
entry $w.midie -width 30 -relief sunken -textvariable rhythm(path_midiplayer)
grid $w.midi $w.midie 
bind .setup.midie  <Return> {focus .config}
.setup.midie  xview moveto 1.0
button $w.save -text "save folder" -width 14 -command set_savefolder
entry $w.savee -width 30 -relief sunken -textvariable rhythm(path_savefolder)
grid $w.save $w.savee 
}

proc make_config_output {} {
global rhythm
global patches
if {[winfo exist .output]} return
set w .output
toplevel .output
label $w.bpm -text "beats/minute" -width 14
scale $w.bpmsca -from 20 -to 120 -variable rhythm(bpm)\
-length 200 -orient hor -width 5
grid $w.bpm $w.bpmsca 
frame $w.6
label $w.beatsperbar -text "beats per bar"
scale $w.beatsbarsca -from 2 -to 4 -variable rhythm(beats_per_bar)\
-length 200 -orient hor -width 5 
grid $w.beatsperbar $w.beatsbarsca 
checkbutton $w.tiebox -variable rhythm(ties) -text ties
scale $w.tiescale -from 0 -to 1.0 -variable rhythm(tiefraction) \
-length 200 -orient hor -width 5 -resolution 0.01
grid $w.tiebox  $w.tiescale 
button $w.voice1 -text "voice 1 parameters" -command set_voice1
button $w.voice2 -text "voice 2 parameters" -command set_voice2
grid $w.voice1 $w.voice2
button $w.leadin -text $patches($rhythm(leader_instrument)) \
 -command leader_prog_dialog -width 20
label $w.leader -text "leader instrument"
grid $w.leader $w.leadin
radiobutton $w.1voice -text "1 voice" -value 0 \
 -variable rhythm(2voice) -command update_play_button
radiobutton $w.2voice -text "2 voices" -value 1 \
 -variable rhythm(2voice) -command update_play_button
grid $w.1voice $w.2voice
}

proc set_voice1 {} {
global rhythm
global patches
if {[winfo exist .prog]}   {.prog.sel configure -text voice1} 
if {[winfo exist .voice1]} return
set w .voice1
toplevel $w
label $w.pitch -text pitch
scale $w.pitchsca -from 24 -to 120 -variable val1\
-length 200 -orient hor -width 5 -command set_pitch1
$w.pitchsca set $rhythm(pitch1)
grid $w.pitch $w.pitchsca
button $w.prog -text $patches($rhythm(rhythm1_instrument)) \
 -command voice1_prog_dialog -width 20
label $w.rhythm -text instrument
grid $w.rhythm $w.prog
}

proc set_voice2 {} {
global rhythm
global patches
if {[winfo exist .prog]}   {.prog.sel configure -text voice2} 
if {[winfo exist .voice2]} return
set w .voice2
toplevel $w
label $w.pitch -text pitch
scale $w.pitchsca -from 24 -to 120 -variable val2\
-length 200 -orient hor -width 5 -command set_pitch2
$w.pitchsca set $rhythm(pitch2)
grid $w.pitch $w.pitchsca
button $w.prog -text $patches($rhythm(rhythm2_instrument)) \
 -command voice2_prog_dialog -width 20
label $w.rhythm -text instrument
grid $w.rhythm $w.prog
}


proc set_pitch1 {val} {
global rhythm
set rhythm(pitch1) $val
}

proc set_pitch2 {val} {
global rhythm
set rhythm(pitch2) $val
}

proc update_play_button {} {
global rhythm
pack forget .control.playmenu
pack forget .control.play
if {$rhythm(2voice)} {
  pack .control.playmenu -after .control.go -side left
 } else {
  pack .control.play -after .control.go -side left
 }
}


proc setpath {path_var} {
global rhythm

    set filedir [file dirname $rhythm($path_var)]
    set openfile [tk_getOpenFile -initialdir $filedir]
    if {[string length $openfile] > 0} {
      set rhythm($path_var) $openfile
      update
    }
}

proc leader_prog_dialog {} {
global instrumentbut
set instrumentbut .output.leadin
if {[info exist .prog]}   {.prog.sel configure -text leadin} 
change_prog_dialog
}


proc voice1_prog_dialog {} {
global instrumentbut
set instrumentbut .voice1.prog
change_prog_dialog
}

proc voice2_prog_dialog {} {
global instrumentbut
set instrumentbut .voice2.prog
change_prog_dialog
}

proc change_prog_dialog {} {
    #create a dialog window for changing the program assignment
    #(channel to instrument mapping). Display a list box with
    #scroll bars with a list of all the General Midi instruments.
    global patches rhythm
    global instrumentbut
    set w .prog
    set instrument 0
    if {[winfo exists .prog] == 0} {
        toplevel $w
        label $w.sel -text $instrumentbut  -relief sunken
        set butlist [split $instrumentbut .]
        if {[string equal [lindex $butlist 2] "prog"]} {
          .prog.sel configure -text [lindex $butlist 1] 
        }  else {.prog.sel configure -text leadin}
        scrollbar $w.scroll -command "$w.list yview"
        listbox $w.list -yscroll "$w.scroll set" -setgrid 1 -height 10\
          -selectmode single
        pack $w.scroll -side right -fill y
        pack $w.sel
        pack $w.list -side top -expand 1 -fill both
        bind $w.list <Button> {select_program [.prog.list nearest %y]}
        for {set i 0} {$i < 128} {incr i} {
            $w.list insert end $patches($i)
        }
    } {
        focus $w
        raise $w .
        set butlist [split $instrumentbut .]
        if {[string equal [lindex $butlist 2] "prog"]} {
          .prog.sel configure -text [lindex $butlist 1] 
        }  else {.prog.sel configure -text leadin}
    }
}

proc select_program {p} {
    global patches rhythm
    global instrumentbut
    $instrumentbut configure -text $patches($p)
    puts [split $instrumentbut .]
    set buttype [lindex [split $instrumentbut .] 2]
    set butlist [split $instrumentbut .]
    if {[string equal [lindex $butlist 2] "prog"]} {
        .prog.sel configure -text [lindex $butlist 1] 
    }  else {.prog.sel configure -text leadin}
    puts $buttype
    if {$buttype == "leadin"} {
       set rhythm(leader_instrument) $p
    } elseif {$buttype == "prog"} {
       set voice [lindex [split $instrumentbut .] 1]
       if {[string equal "voice1" $voice]} {set rhythm(rhythm1_instrument) $p
      }  else {set rhythm(rhythm2_instrument) $p}
    }
}



proc write_ini_file {filename} {
global rhythm
if {$rhythm(lockconfig) > 1} return
set handle [open $filename "w"]
puts $handle "path_abcm2ps $rhythm(path_abcm2ps)"
puts $handle "path_abc2midi $rhythm(path_abc2midi)"
puts $handle "path_gs $rhythm(path_gs)"
puts $handle "path_midiplayer $rhythm(path_midiplayer)"
puts $handle "bpm $rhythm(bpm)"
puts $handle "beats_per_bar  $rhythm(beats_per_bar)"
puts $handle "ties $rhythm(ties)"
puts $handle "leader_instrument $rhythm(leader_instrument)"
puts $handle "rhythm1_instrument $rhythm(rhythm1_instrument)"
puts $handle "rhythm2_instrument $rhythm(rhythm2_instrument)"
puts $handle "pitch1 $rhythm(pitch1)"
puts $handle "pitch2 $rhythm(pitch2)"
puts $handle "rhythmpitch $rhythm(pitch1)"
puts $handle "path_savefolder $rhythm(path_savefolder)"
puts $handle "tiefraction $rhythm(tiefraction)"
puts $handle "2voice $rhythm(2voice)"
puts $handle "lockconfig $rhythm(lockconfig)"
puts $handle "nosave $rhythm(nosave)"
puts $handle "temp_path $rhythm(temp_path)"
puts $handle "repeatability $rhythm(repeatability)"
close $handle
}

proc read_tkrhythm_ini {} {
global rhythm
set handle [open tkrhythm.ini r]
while {[gets $handle line] >= 0} {
   set error_return [catch {set n [llength $line]} error_out]
   if {$error_return} continue
   set contents ""
   set param [lindex $line 0]
   for {set i 1} {$i < $n} {incr i} {
     set contents [concat $contents [lindex $line $i]]
   }
#if param is not already a member of the trainer array
#then we ignore it. This prevents the trainer array filling up
#with obsolete parameters used in older versions of the program.
   set member [array names rhythm $param]
   if [llength $member] {set rhythm($param) $contents }
    }
close $handle
set len [string length $rhythm(temp_path)]
if {$len > 1} {
  set lastchar [string index $rhythm(temp_path) [incr len -1]]
  if {[string equal $lastchar "/"]} return
  append rhythm(temp_path) /
  }
}

proc report_executable_call {} {
global result result2
tk_messageBox -type ok -icon info -message $result
tk_messageBox -type ok -icon info -message $result2
}

bind . <c> report_executable_call


if {[file exist tkrhythm.ini]} read_tkrhythm_ini

set cmn [image create photo -width 600 -height 500]

set w .control
set timesigtype 0
frame $w
radiobutton $w.simple -text "simple" -variable timesigtype -value 0 \
   -command switch_type
radiobutton $w.compound -text "compound" -variable timesigtype -value 1\
   -command switch_type

button $w.cfg -text configure -command make_config_output
menubutton $w.cfg2 -text configure  -menu $w.cfg2.item
menu $w.cfg2.item -tearoff 0
$w.cfg2.item add command -label setup -command make_config_setup
$w.cfg2.item add command -label output -command make_config_output
button $w.go -text "create " -command make_abc_file
button $w.play -text "play" -command "play_exercise 0" -state disable
menubutton $w.playmenu -text "play" -menu $w.playmenu.item -state disable
menu $w.playmenu.item -tearoff 0 
$w.playmenu.item add command -label "voice 1" -command {play_exercise 1}
$w.playmenu.item add command -label "voice 1 & 2" -command {play_exercise 0}
button $w.save -text "save" -command save_pdf_midi -state disable
button $w.help -text ? -command {tk_messageBox -type ok -icon info -message "$VERSION http://ifdo.ca/~seymour/tkrhythm"} 
if {$rhythm(lockconfig)} {set cfg cfg} else {set cfg cfg2}
if {$rhythm(2voice)} {set play playmenu} else {set play play}
pack $w.simple $w.compound $w.$cfg $w.go $w.$play $w.save $w.help -side left
pack $w

read_patch_names
make_simple_interface
make_compound_interface
pack .rhythmselector2 

make_temp_folder 


#source randseed.tcl


proc ValidInt {val} {
 return [ expr {[string is integer $val] || [string match {[-+]} $val]}]
}

proc Validchar {val} {
 if {[string length $val] > 4} {return 0}
 return [string is alnum $val] 
 }



proc n2alpha {n} {
#converts a number between 0 and 62 to an alphanumeric
#character 
if {$n < 10} {
  return [format "%c" [expr 48+$n]]
  }
if {$n < 36} {
  return [format "%c" [expr 55+$n]]
  }
return [format "%c" [expr 61 + $n]]
}



proc n2string {n} {
#converts a number to a 4 character alphanumeric string 
  set s ""
  set m [expr $n % 62]
  set s $s[n2alpha $m]
  set n [expr $n / 62]
  set m [expr $n % 62]
  set s $s[n2alpha $m]
  set n [expr $n / 62]
  set m [expr $n % 62]
  set s $s[n2alpha $m]
  set n [expr $n / 62]
  set m [expr $n % 62]
  set s $s[n2alpha $m]
  return $s
  }

proc string2n {s} {
# converts a string to a list of ascii values
set len [string length $s]
binary scan $s c$len nlist
#puts $nlist
set n 0
for {set l 3} {$l >= 0} {incr l -1} {
  set code [lindex $nlist $l]
  set val [ascii2num $code]
  #puts "$code $val"
  set n [expr $n*62 + $val]
  }
return $n
}

proc ascii2num {n} {
#converts an alphanumeric character to a number between 0 and 61
if {$n <58} {return [expr $n - 48]}
if {$n <91} {return [expr $n - 55]}
if {$n <123} {return [expr $n - 61]}
}


set maxval [expr 62*62*62*62]

proc make_random_seed {} {
global maxval rseed
set n [expr int($maxval*rand())]
set rseed [n2string $n]
#puts $rseed
activate_random_seed
}

proc activate_random_seed {} {
global rseed
set n [string2n $rseed]
expr srand($n)
}

proc reset_random_seed {} {
global rseed
set n [string2n $rseed]
expr srand($n)
focus .f
}

if {$rhythm(repeatability)} {
  set ww {-width 8}
  frame .f
  pack .f
  eval button .f.rseedbut -text "random " $ww -command make_random_seed
  entry .f.rseed -width 8 -textvariable rseed -vcmd {Validchar %P} \
      -validate all
  eval button .f.rseedaction -text "reset" $ww -command reset_random_seed
  make_random_seed
  bind .f.rseed  <Return> {
     reset_random_seed}
  grid .f.rseedbut .f.rseed .f.rseedaction
  }



make_random_seed
#end or source randseed.tcl


#set out [open glyph.abc w]
#make_4_4_header
#make_all_glyphs
#close $out

